// *********************************************************************************
//   Java application to perform G-LoSA search against PDB local structure library
//
//                developed by Hui Sun Lee (huisun.cadd@gmail.com)
// *********************************************************************************

import java.io.*;
import java.text.DecimalFormat;



public class GLoSASearch{
	static String mDir_GLoSA_exe = null;
	static double mCutoffGAScore = 1.0;
		
	
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 1 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "Usage: GLoSASearch" );
			System.out.println( "        -opts  [input parameter file]" );
			System.out.println( "                   *please refer to \'glosa_search.inp\' for the format of input parameter file" );

			return;
		}
		
		String mRefStruct = null;
		String mRefStructCF = null;
		String mRefSurface = null;
		String mRefSurfaceCF = null;
		String mDirLib = null;
		String mList = null;
		String mDirLibCF = null;
		
		String [] mParameters = new String[9];
		
		
		
		
		DecimalFormat format = new DecimalFormat( "####.###" );   //%8.3f
		format.setMinimumFractionDigits(3); 
		format.setMaximumFractionDigits(3); 
		
		
		FileInputStream parameter_stream;
		BufferedReader parameter_reader;	
		String parameter_line;
		
		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;
		
		FileInputStream matrix_stream;
		BufferedReader matrix_reader;	
		String matrix_line;

		DataOutputStream out_score;		
		DataOutputStream out_matrix;		

		
		String mFitStruct = "";
		String mFitStructCF = "";


		String line;
		String [] array;
		double ga_score;

		
		
		
		

		try{
			// *** parameter setting for G-LoSA running ***
			parameter_stream = new FileInputStream( new File( args[0] ) );
			parameter_reader = new BufferedReader( new InputStreamReader( parameter_stream ) );	
			
			int parameter_index = -1;
			
			while( ( parameter_line = parameter_reader.readLine()) != null ){
				if( parameter_line.charAt(0) == '>' ){
					parameter_index++;
					mParameters[parameter_index] = parameter_line.substring( 1, parameter_line.length() ).trim();
				}
			}
			
			parameter_reader.close();
			parameter_stream.close();
			
			mRefStruct = mParameters[0];
			mRefStructCF = mParameters[1];
			mRefSurface = mParameters[2];
			mRefSurfaceCF = mParameters[3];
			mDirLib = mParameters[4];
			mList = mParameters[5];
			mDirLibCF = mParameters[6];
			mDir_GLoSA_exe = mParameters[7];
			mCutoffGAScore = Double.parseDouble( mParameters[8] );
			
			
			
			
			
			
			
			out_score = new DataOutputStream( new FileOutputStream( new File( "score.txt" ) ) );	
			out_matrix = new DataOutputStream( new FileOutputStream( new File( "list_matrix.txt" ) ) );
			
			
			
			list_stream = new FileInputStream( new File( mList ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	
			
			while( ( list_line = list_reader.readLine()) != null ){
				System.out.println( list_line );
				
				mFitStruct = list_line.trim();
				mFitStructCF = list_line.substring( 0, list_line.length()-4 ) + "-cf.pdb";



                // *** run G-LoSA ***
                String result = runLSS( mRefStruct, mRefStructCF, mRefSurface, mRefSurfaceCF, mDirLib + mFitStruct, mDirLibCF + mFitStructCF );
                
                result = result.trim();
                
                if( result.length() == 0 ){
					ga_score = 0;
				}
				else if( result.equalsIgnoreCase( "-nan" ) ){
					ga_score = 0;
				}
				else{
					ga_score = Double.parseDouble( result );
				}						
						
				out_score.writeBytes( mFitStruct + "\t" + ga_score + "\n" );	
				
				
				
				// *** write rotation matrix ***		
				if( ga_score >= mCutoffGAScore ){
					out_matrix.writeBytes( ">" + list_line + "\n" );
					
					matrix_stream = new FileInputStream( new File( "matrix.txt" ) );
					matrix_reader = new BufferedReader( new InputStreamReader( matrix_stream ) );	
					
					matrix_reader.readLine();
					matrix_reader.readLine();
					matrix_line = matrix_reader.readLine();
					out_matrix.writeBytes( matrix_line.substring( 4, matrix_line.length() ) + "\n" );
					matrix_line = matrix_reader.readLine();
					out_matrix.writeBytes( matrix_line.substring( 4, matrix_line.length() ) + "\n" );
					matrix_line = matrix_reader.readLine();
					out_matrix.writeBytes( matrix_line.substring( 4, matrix_line.length() ) + "\n" );
					
					matrix_reader.close();
					matrix_stream.close();
				}				
			}
			
			list_reader.close();
			list_stream.close();	
			
			
			
			out_matrix.close();
			out_score.close();	
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}		
	}	






	private static String runLSS( String ref_struct, String ref_struct_cf, String ref_surface, String ref_surface_cf, String fit_struct, String fit_struct_cf ){
		FileInputStream input_stream;
		BufferedReader input_reader;	
		String input_line;

		Runtime rt = Runtime.getRuntime();
		Process pr;
		LineNumberReader output_log;
		String pr_line;
		String command;

		String scoring_out = "";
		String [] array;

		try{
			if( ref_surface.charAt(0) == 'n' && ref_surface_cf.charAt(0) == 'n' )
				command = mDir_GLoSA_exe + " -s1 " + ref_struct + " -s1cf " + ref_struct_cf + " -s2 " + fit_struct + " -s2cf " + fit_struct_cf + " -o 2";
			else
				command = mDir_GLoSA_exe + " -s1 " + ref_struct + " -s1cf " + ref_struct_cf + " -surf 1 -surf1 " + ref_surface + " -surf1cf " + ref_surface_cf + " -s2 " + fit_struct + " -s2cf " + fit_struct_cf + " -o 2";
	
			pr = rt.exec( command );
			output_log = new LineNumberReader( new InputStreamReader( pr.getInputStream() ) );	
			
			while( ( pr_line = output_log.readLine() ) != null ){
				if( pr_line.startsWith( "GA-score" ) ){
					array = pr_line.split( ":" );
					scoring_out = array[1].trim();
					break;
				}
			}
						
			pr.waitFor();
			output_log.close();
			pr.getInputStream().close();
			pr.destroy();  				
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}	

		return scoring_out;
	}
}
