// ********************************************************************************************
//   Java application to generate a PyMol script to display target and template structures
//
//                     developed by Hui Sun Lee (huisun.cadd@gmail.com)
// ********************************************************************************************

import java.io.*;



public class GeneratePyMolScript{
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 4 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "GeneratePyMolScript" );
			System.out.println( "           -opts [target protein structure)]" );
			System.out.println( "                 [templates list]" );
			System.out.println( "                 [number of templates to be displayed]" );
			System.out.println( "                 [directory where aligned template structures are saved]" );

			return;
		}		
		
		
		String mStructTarget = args[0];
		String mListTemplates = args[1];
		int mTopN = Integer.parseInt( args[2] );
		String mDirTemplates = args[3];
		
		
		
		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;		

		DataOutputStream out;		

		
		String [] array;

		

		try{
			String path_target_protein = "";
			String file_target_protein = "";
			
			array = mStructTarget.split( "/" );
			
			if( array.length == 1 ){
				path_target_protein = "./";
				file_target_protein = array[0];
			}
			else{
				for( int i = 0 ; i < ( array.length - 1 ) ; i++ )
					path_target_protein += array[i] + "/";
				
				file_target_protein = array[array.length - 1];
			}
			
			
			out = new DataOutputStream( new FileOutputStream( new File( "view_struct.pml" ) ) );
			
			out.writeBytes( "cmd.bg_color(\"white\")" );
			out.writeBytes( "\n" );
			out.writeBytes( "cmd.space(\"cmyk\")" );
			out.writeBytes( "\n" );
			out.writeBytes( "util.performance(0)" );
			out.writeBytes( "\n" );
			out.writeBytes( "util.ray_shadows(\"none\")" );
			out.writeBytes( "\n" );
			out.writeBytes( "set stick_radius,0.15" );
			out.writeBytes( "\n" );
			out.writeBytes( "set line_width,2.0" );
			out.writeBytes( "\n" );
			out.writeBytes( "cmd.set(\'cartoon_fancy_helices\',\'1\',quiet=0)" );
			out.writeBytes( "\n" );
			
			out.writeBytes( "load " );
			out.writeBytes( path_target_protein + file_target_protein );
			out.writeBytes( ", " );
			out.writeBytes( file_target_protein );
			out.writeBytes( "\n" );	
			out.writeBytes( "cmd.color(\"green\",\"byobject(" );
			out.writeBytes( file_target_protein );
			out.writeBytes( ")\")" );
			out.writeBytes( "\n" );  			
			out.writeBytes( "cmd.show(\"cartoon\",\"byobject(" );
			out.writeBytes( file_target_protein );
			out.writeBytes( ")\")" );
			out.writeBytes( "\n" );
			

			
			
			list_stream = new FileInputStream( new File( mListTemplates ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	
			
			int num_template = 0;

			while( ( list_line = list_reader.readLine()) != null ){
				num_template++;
				
				if( num_template > mTopN )
					break;
					
				array = list_line.split( "\t" );
				String template_bsres = array[0];
				array = template_bsres.split( "-" );
				String template_lig = array[0] + "-lig-" + array[2];
				
				out.writeBytes( "load " );
				out.writeBytes( mDirTemplates + template_bsres );
				out.writeBytes( ", " );
				out.writeBytes( template_bsres );
				out.writeBytes( "\n" );	
				out.writeBytes( "cmd.color(\"blue\",\"byobject(" );
				out.writeBytes( template_bsres );
				out.writeBytes( ")\")" );
				out.writeBytes( "\n" );  
				out.writeBytes( "cmd.show(\"sticks\",\"byobject(" );
				out.writeBytes( template_bsres );
				out.writeBytes( ")\")" );
				out.writeBytes( "\n" );  
				out.writeBytes( "util.cnc(\"" );
				out.writeBytes( template_bsres );
				out.writeBytes( "\", _self=cmd)" );
				out.writeBytes( "\n" );  				
				out.writeBytes( "cmd.disable(\"" );
				out.writeBytes( template_bsres );	
				out.writeBytes( "\")" );	
				out.writeBytes( "\n" );
				
				
				out.writeBytes( "load " );
				out.writeBytes( mDirTemplates + template_lig );
				out.writeBytes( ", " );
				out.writeBytes( template_lig );
				out.writeBytes( "\n" );	
				out.writeBytes( "cmd.color(\"blue\",\"byobject(" );
				out.writeBytes( template_lig );
				out.writeBytes( ")\")" );
				out.writeBytes( "\n" );   								
				out.writeBytes( "cmd.show(\"sticks\",\"byobject(" );
				out.writeBytes( template_lig );
				out.writeBytes( ")\")" );
				out.writeBytes( "\n" );
				out.writeBytes( "util.cnc(\"" );
				out.writeBytes( template_lig );
				out.writeBytes( "\", _self=cmd)" );
				out.writeBytes( "\n" );  				
				out.writeBytes( "cmd.disable(\"" );
				out.writeBytes( template_lig );	
				out.writeBytes( "\")" );	
				out.writeBytes( "\n" );
			}

			list_reader.close();
			list_stream.close();	
			
			
			out.writeBytes( "cmd.zoom(\"all\",animate=-1)" );	
			out.writeBytes( "\n" ); 
			
			out.close();
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}		
	}
}