// ********************************************************************
//   Java application to generate G-LoSA aligned template structures
//
//            developed by Hui Sun Lee (huisun.cadd@gmail.com)
// ********************************************************************

import java.io.*;
import java.text.DecimalFormat;
import java.util.Hashtable;
import java.util.StringTokenizer;



public class GetAlignedTemplates{
	static DecimalFormat format;


	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 4 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "Usage: GetAlignedTemplates" );
			System.out.println( "          -opts  [template list]" );
			System.out.println( "                 [directory of structure library]" );
			System.out.println( "                 [rotation matrix file (list_matrix.txt)]" );
			System.out.println( "                 [directory where aligned template structures will be saved]" );
			return;
		}
		
		String mListTemplates = args[0];
		String mDirLib = args[1];
		String mListMatrix = args[2];
		String mDirTarget  = args[3];
		
		
		
		
		// *** Number format ***
		format = new DecimalFormat( "####.###" );   //%8.3f
		format.setMinimumFractionDigits(3); 
		format.setMaximumFractionDigits(3); 
		
		
		
		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;
		

		String [] array;
		String value = null;
		StringTokenizer st = null;
		double [] t = new double[3];
		double [][] u = new double[3][3];

		
		
		
		

		try{
			// *** read template names ***
			Hashtable<String,String> hTemplates = new Hashtable<String,String>();
			
			list_stream = new FileInputStream( new File( mListTemplates ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	
			
			while( ( list_line = list_reader.readLine()) != null ){
				array = list_line.split( "\t" );
				
				hTemplates.put( array[0], "1" );
			}
			
			list_reader.close();
			list_stream.close();	
			
			
			
			
			// *** read rotation matrices list ***
			list_stream = new FileInputStream( new File( mListMatrix ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	
			
			while( ( list_line = list_reader.readLine()) != null ){
				if( list_line.charAt(0) == '>' ){
					String bsres = list_line.substring( 1, list_line.length() );
					value = hTemplates.get( bsres );
					
					if( value != null ){
						System.out.println( bsres );
							
						st = new StringTokenizer( list_reader.readLine() );
						t[0] = Double.parseDouble( st.nextToken() );
						u[0][0] = Double.parseDouble( st.nextToken() );
						u[0][1] = Double.parseDouble( st.nextToken() );
						u[0][2] = Double.parseDouble( st.nextToken() );

						st = new StringTokenizer( list_reader.readLine() );
						t[1] = Double.parseDouble( st.nextToken() );
						u[1][0] = Double.parseDouble( st.nextToken() );
						u[1][1] = Double.parseDouble( st.nextToken() );
						u[1][2] = Double.parseDouble( st.nextToken() );

						st = new StringTokenizer( list_reader.readLine() );
						t[2] = Double.parseDouble( st.nextToken() );
						u[2][0] = Double.parseDouble( st.nextToken() );
						u[2][1] = Double.parseDouble( st.nextToken() );
						u[2][2] = Double.parseDouble( st.nextToken() );		
						
						array = bsres.split( "-" );
						String lig = array[0] + "-lig-" + array[2];
						
						getSuperposedStructure( t, u, mDirLib + bsres, mDirTarget + bsres );
						getSuperposedStructure( t, u, mDirLib + lig, mDirTarget + lig );
					}
				}
			}
			
			list_reader.close();
			list_stream.close();	
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}		
	}
	
	
	
	
	
	public static String makePDBline( String line, double[] R ){	
		String new_line = "";
		String line_1 = line.substring( 0, 30 );
		String line_2 = line.substring( 54, line.length() );

		for( int i = 0 ; i < 3 ; i++ ){
			if( format.format( R[i] ).equalsIgnoreCase( "-0.000" ) )
				R[i] = Double.parseDouble( format.format( 0.000 ) );
			else
				R[i] = Double.parseDouble( format.format( R[i] ) );

			if( R[i] >= 0 ){
				if( R[i] / 10 < 1 )
					new_line = new_line + "   " + format.format( R[i] );
				else if( R[i] / 10 >= 1 && R[i] / 10 < 10 )
					new_line = new_line + "  " + format.format( R[i] );
				else
					new_line = new_line + " " + format.format( R[i] );
			}
			else{
				if( R[i] / 10 > -1 )
					new_line = new_line + "  " + format.format( R[i] );
				else if( R[i] / 10 <= -1 && R[i] / 10 > -10 )
					new_line = new_line + " " + format.format( R[i] );
				else
					new_line = new_line + format.format( R[i] );
			}
		}	

		new_line = line_1 + new_line + line_2;

		return new_line;
	}
	
	
	
	
	
	public static void getSuperposedStructure( double [] t, double [][] u, String struct_in, String struct_out ){
		FileInputStream struct_stream;
		BufferedReader struct_reader;	
		String struct_line;
		
		DataOutputStream out;	
		
		double [] r = new double[3];
		double [] r_prime = new double[3];
		
		try{
			struct_stream = new FileInputStream( new File( struct_in ) );				
			struct_reader = new BufferedReader( new InputStreamReader( struct_stream ) );	
			
			out = new DataOutputStream( new FileOutputStream( new File( struct_out ) ) );	
			

			while( ( struct_line = struct_reader.readLine() ) != null ){
				if( struct_line.startsWith( "ATOM" ) || struct_line.startsWith( "HETATM" ) ){
					r[0] = Double.parseDouble( struct_line.substring( 30, 38 ).trim() );
					r[1] = Double.parseDouble( struct_line.substring( 38, 46 ).trim() );
					r[2] = Double.parseDouble( struct_line.substring( 46, 54 ).trim() );	

					r_prime[0] = t[0] + u[0][0]*r[0] + u[0][1]*r[1] + u[0][2]*r[2];
					r_prime[1] = t[1] + u[1][0]*r[0] + u[1][1]*r[1] + u[1][2]*r[2];
					r_prime[2] = t[2] + u[2][0]*r[0] + u[2][1]*r[1] + u[2][2]*r[2];

					out.writeBytes( makePDBline( struct_line, r_prime ) );
					out.writeBytes( "\n" );
				}
				else{
					out.writeBytes( struct_line );
					out.writeBytes( "\n" );
				}
			}
			
			
			out.close();

			struct_reader.close();
			struct_stream.close();			
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}
	}
}
