// **************************************************************************************************************
//   Java application to generate a html file (index.html) to retrieve annotation information on each template
//
//                                developed by Hui Sun Lee (huisun.cadd@gmail.com)
// **************************************************************************************************************

import java.io.*;
import java.text.DecimalFormat;



public class GetAnnotations{
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 2 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "GeneratePyMolScript" );
			System.out.println( "           -opts [templates list]" );
			System.out.println( "                 [number of templates to be displayed]" );

			return;
		}		
		
		
		String mListTemplates = args[0];
		int mTopN = Integer.parseInt( args[1] );
		
		
		
		DecimalFormat format = new DecimalFormat( "####.###" );   //%8.3f
		format.setMinimumFractionDigits(3); 
		format.setMaximumFractionDigits(3); 
		
		
		
		
		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;		

		DataOutputStream out;		

		
		String [] array;

		

		try{
			out = new DataOutputStream( new FileOutputStream( new File( "annotation.html" ) ) );
			
			out.writeBytes( "<html>" + "\n" );
			out.writeBytes( "<head><title>Annotations</title></head>" + "\n" );
			out.writeBytes( "<body bgcolor=\"#FFFFFF\">" + "\n" );
			out.writeBytes( "<font face=\"Times New Roman\">" + "\n" );
			
			out.writeBytes( "<div>" + "\n" );
  			out.writeBytes( "<h2>Annotation Information on Top " + mTopN + " Templates</h2>" + "\n" );
   			out.writeBytes( "</div>" + "\n" );
   
			out.writeBytes( "<div>" + "\n" );
  			out.writeBytes( "<table border=\"1\" style=\"font-size:17px;border-collapse:collapse\">" + "\n" );
  			out.writeBytes( "<tr style=\"background:#82CAFA\">" + "\n" );
  			out.writeBytes( "<td width=\"30\" align=\"center\">#</td>" + "\n" );
  			out.writeBytes( "<td width=\"200\" align=\"center\">Template binding site</td>" + "\n" );
  			out.writeBytes( "<td width=\"120\" align=\"center\">GA-score</td>" + "\n" );
  			out.writeBytes( "<td width=\"150\" align=\"center\">PDB ID</td>" + "\n" );
  			out.writeBytes( "<td width=\"60\" align=\"center\">Chain</td>" + "\n" );
  			out.writeBytes( "<td width=\"120\" align=\"center\">Annotation</td>" + "\n" );
			out.writeBytes( "</tr>" + "\n" );		
					
			list_stream = new FileInputStream( new File( mListTemplates ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	
			
			int num_template = 0;

			while( ( list_line = list_reader.readLine()) != null ){
				num_template++;
				
				array = list_line.split( "\t" );
				String template_bsres = array[0];
				String pdb_id = template_bsres.substring( 0 , 4 ).toUpperCase();
				double gascore = Double.parseDouble( array[1] );
				
				if( num_template > mTopN )
					break;
					
				out.writeBytes( "<tr>" + "\n" );
  				out.writeBytes( "<td align=\"center\">" + num_template + "</td>" + "\n" );
  				out.writeBytes( "<td align=\"center\">" + template_bsres + "</td>" + "\n" );
  				out.writeBytes( "<td align=\"center\">" + format.format( gascore ) + "</td>" + "\n" );
  				out.writeBytes( "<td align=\"center\"><a href=\"http://www.rcsb.org/pdb/explore/explore.do?structureId=" + pdb_id + "\" target=\"_blank\">" + pdb_id + "</a></td>" + "\n" );
  				out.writeBytes( "<td align=\"center\">" + template_bsres.charAt( 4 ) + "</td>" + "\n" );
  				out.writeBytes( "<td align=\"center\"><a href=\"http://www.rcsb.org/pdb/explore/macroMoleculeData.do?structureId=" + pdb_id + "\" target=\"_blank\">" + "link" + "</a></td>" + "\n" );
				out.writeBytes( "</tr>" + "\n" );		
			}

			list_reader.close();
			list_stream.close();	
			
			
			
			out.writeBytes( "</html>" + "\n" );
			
			
			out.close();
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}		
	}
}