// ****************************************************************************************
//   Java application to remove structural redundancy in binding site structure library
//   
//                  developed by Hui Sun Lee (huisun.cadd@gmail.com)
// **************************************************************************************** 

import java.io.*;
import java.util.Vector;
import java.util.Hashtable;



public class RemoveRedundantLigandBS{
	static String mList;
	static String mDirLib;
	static String mBCX;    
	static double mCutoffLigSimilarity;  
	
	
		
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 5 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "Usage: RemoveRedundantLigandBS" );
			System.out.println( "            -opts  [list of binding site structures with resolution property]" );
			System.out.println( "                          example:  " );
			System.out.println( "                            101mA-BS-1.pdb  2.07" );
			System.out.println( "                            102lA-BS-1.pdb  1.74" );	
			System.out.println( "                            102mA-BS-1.pdb  1.84" );
			System.out.println( "                            ...     " );	
			System.out.println( "                           *X-ray resolution property can be added using AddPropertyResolution in G-LoSA Toolkit" );	
			System.out.println( "                   [directory where library ligand structures are saved]" );
			System.out.println( "                   [bc-X.out (RCSB protein clustering result at X% sequence identity)]" );	
			System.out.println( "                       *bc-X.out files is available at http://www.rcsb.org/pdb/statistics/clusterStatistics.do" );
			System.out.println( "                   [cutoff ligand similarity score (Tanimoto coefficient: 0 ~ 1)]" );
			System.out.println( "                   [output non-redundant file name]" );
				
			return;
		}		
		
		
		
		mList = args[0];
		mDirLib = args[1];
		mBCX = args[2];    // bc-50.out
		mCutoffLigSimilarity = Double.parseDouble( args[3] );   // 0.7
		String mOutput = args[4];
		
		
		
		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;

		DataOutputStream out = null;	

		
		Vector<String> vBS = new Vector<String>();
		Hashtable<String,String> hHomolgousProteinChains = new Hashtable<String,String>();
		String [] array;
		

				

		try{	
			if( mDirLib.charAt( mDirLib.length() - 1 ) != '/' )
				mDirLib = mDirLib + "/";
				
				
				
				
			// ***************************************
			//    read binding site structures list
			// ***************************************
			
			list_stream = new FileInputStream( new File( mList ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	
			
			while( ( list_line = list_reader.readLine()) != null ){
				vBS.add( list_line );
			}
			
			list_reader.close();
			list_stream.close();
			
			
			
			
			
			// *******************
			//    read bc-X.out
			// *******************
			 
			list_stream = new FileInputStream( new File( mBCX ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	
			
			int bc_index = 0;
			
			while( ( list_line = list_reader.readLine()) != null ){
				bc_index++;
				
				array = list_line.split( " " );
				
				for( int i  = 0 ; i < array.length ; i++ ){
					hHomolgousProteinChains.put( array[i], "" + bc_index );
				}
			}
			
			list_reader.close();
			list_stream.close();
			
			
			
			
			
			
			
			// =============
			//   Variables
			// =============
			
			int num_BS = 0;
			
			Vector<String> vTemp = new Vector<String>();
			Vector<String> vBS_nr = new Vector<String>();
			
			String bs_prev = "XXXXX";
			String bs_curr = "XXXXX";
			String bs_next = "XXXXX";
			
			String bs_i = null;
			String bs_j = null;
			
			double resol_i = 5.0;
			double resol_j = 5.0;
			
			String lig_i = null;
			String lig_j = null;
			
			String value_i = null;
			String value_j = null;
			
			
			
			
			
			// *****************************************************************************
			//    step 1: remove redundant binding site structures within individual PDBs
			// *****************************************************************************
			
			System.out.println( "\nstep 1: remove redundant binding site structures within individual PDBs\n" );
			
			num_BS = vBS.size();
			
			int index = 0;
			
			while( true ){
				if( index == ( num_BS - 1 ) )
					break;
					
				vTemp.removeAllElements();
				
				bs_curr = vBS.elementAt(index);
				vTemp.add( bs_curr );
				
				System.out.println( bs_curr );
				
				while( true ){
					index++;
					bs_next = vBS.elementAt(index);
						
					if( bs_curr.substring( 0, 4 ).equalsIgnoreCase( bs_next.substring( 0, 4 ) ) ){
						vTemp.add( bs_next );
						
						if( index == ( num_BS - 1 ) ){
							filterVector( hHomolgousProteinChains, vTemp );
							addVector( vTemp, vBS_nr );
							break;
						}
					}
					else{
						filterVector( hHomolgousProteinChains, vTemp );
						addVector( vTemp, vBS_nr );		
						
						if( index == ( num_BS - 1 ) )	
							vBS_nr.add( bs_next );
			
						break;
					}	
				}
			}
			
			
			vBS.removeAllElements();
			
			
			
			
			
			// *****************************************************************************
			//    step 2: remove redundant binding site structures between different PDBs
			// *****************************************************************************
			
			System.out.println( "\nstep 2: remove redundant binding site structures between different PDBs\n" );
			
			num_BS = vBS_nr.size();
			
			for( int i = 0 ; i < num_BS ; i++ ){
				array = vBS_nr.elementAt(i).split( "\t" );
				bs_i = array[0];
				resol_i = Double.parseDouble( array[1] );
				value_i = hHomolgousProteinChains.get( bs_i.substring( 0, 4 ).toUpperCase() + "_" + bs_i.charAt(4) );
				
				System.out.println( bs_i );
				
				for( int j = i+1 ; j < num_BS ; j++ ){
					array = vBS_nr.elementAt(j).split( "\t" );
					bs_j = array[0];
					resol_j = Double.parseDouble( array[1] );
					value_j = hHomolgousProteinChains.get( bs_j.substring( 0, 4 ).toUpperCase() + "_" + bs_j.charAt(4) );
					
					if( ( value_i != null && value_j != null ) && ( Integer.parseInt( value_i ) == Integer.parseInt( value_j ) ) ){
						array = bs_i.split( "-" );
						lig_i = array[0] + "-lig-" + array[2];
							
						array = bs_j.split( "-" );
						lig_j = array[0] + "-lig-" + array[2];
							
						if( getLigandSimilarity( mDirLib + lig_i, mDirLib + lig_j ) > mCutoffLigSimilarity ){
							if( resol_i <= resol_j ){
								vBS_nr.removeElementAt(j);
								j--;
								num_BS--;
							}
							else{
								vBS_nr.removeElementAt(i);
								i--;
								num_BS--;
								break;
							}
						}
					}
				}
			}
			
			
			
			
			out = new DataOutputStream( new FileOutputStream( new File( mOutput ) ) );

			for( int i = 0 ; i < vBS_nr.size() ; i++ ){
				array = vBS_nr.elementAt(i).split( "\t" );
				out.writeBytes( array[0] + "\n" );
			}
			
			out.close();
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}
	}
	
	
	
	
	
	
	
	public static void filterVector( Hashtable<String,String> hHomolgousProteinChains, Vector<String> vBS ){
		String bs_i = null;
		String bs_j = null;
			
		String lig_i = null;
		String lig_j = null;
			
		String value_i = null;
		String value_j = null;
		
		String [] array;
			
		for( int i = 0 ; i < vBS.size() ; i++ ){
			array = vBS.elementAt(i).split( "\t" );
			bs_i = array[0];
			value_i = hHomolgousProteinChains.get( bs_i.substring( 0, 4 ).toUpperCase() + "_" + bs_i.charAt(4) );
				
			for( int j = i+1 ; j < vBS.size() ; j++ ){
				array = vBS.elementAt(j).split( "\t" );
				bs_j = array[0];
				value_j = hHomolgousProteinChains.get( bs_j.substring( 0, 4 ).toUpperCase() + "_" + bs_j.charAt(4) );
					
				if( ( value_i != null && value_j != null ) && ( Integer.parseInt( value_i ) == Integer.parseInt( value_j ) ) ){
					array = bs_i.split( "-" );
					lig_i = array[0] + "-lig-" + array[2];
							
					array = bs_j.split( "-" );
					lig_j = array[0] + "-lig-" + array[2];
							
					if( getLigandSimilarity( mDirLib + lig_i, mDirLib + lig_j ) > mCutoffLigSimilarity ){
						vBS.removeElementAt(j);
						j--;
					}
				}
			}
		}
	}
	
	
	
	
	
	
	// **************************************************************************************
	//   calculate structure similarity between ligands using a path-based FP2 fingerprint
	// **************************************************************************************
	public static double getLigandSimilarity( String lig_1, String lig_2 ){
		Runtime rt = Runtime.getRuntime();
		Process pr;
		LineNumberReader output_log;
		String pr_line;
		String command;
		String scoring_out = "";
		String [] array;
		double score = 0;
		
		
		try{
			command = "babel " + lig_1 + " " + lig_2 + " -ofpt ";

			pr = rt.exec( command );			
			output_log = new LineNumberReader( new InputStreamReader( pr.getInputStream() ) );	

			while( ( pr_line = output_log.readLine() ) != null ){	
				array = pr_line.split( "   " );
				if( ( array.length > 1 ) && array[1].trim().startsWith( "Tanimoto" ) ){
					array = array[1].trim().split( "=" );
					score = Double.parseDouble( array[1].trim() );
				}
			}				
			
			pr.waitFor();
			output_log.close();
			pr.getInputStream().close();
			pr.destroy();  	
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}	
		
		return score;
	}
	
	
	
	
	
	
	public static void addVector( Vector<String> vTemp, Vector<String> vBS_nr ){
		for( int i = 0 ; i < vTemp.size() ; i++ )
			vBS_nr.add( vTemp.elementAt(i) );
	}
}