// ****************************************************
//               Class StructureHandler
//   
//   developed by Hui Sun Lee (huisun.cadd@gmail.com)
// *****************************************************

import java.io.*;
import java.util.Vector;
import java.text.DecimalFormat;



public class StructureHandler{
	DecimalFormat format;

	public StructureHandler(){
		format = new DecimalFormat( "####.###" );   //%8.3f
		format.setMinimumFractionDigits(3); 
		format.setMaximumFractionDigits(3);
	}



	
	public double getDistance( double[] r1, double[] r2 ){
		return Math.sqrt( ( r1[0] - r2[0] ) * ( r1[0] - r2[0] ) + ( r1[1] - r2[1] ) * ( r1[1] - r2[1] ) + ( r1[2] - r2[2] ) * ( r1[2] - r2[2] ) );
	}




	public double getDistance( Atom r1, Atom r2 ){
		return Math.sqrt( ( r1.R[0] - r2.R[0] ) * ( r1.R[0] - r2.R[0] ) + ( r1.R[1] - r2.R[1] ) * ( r1.R[1] - r2.R[1] ) + ( r1.R[2] - r2.R[2] ) * ( r1.R[2] - r2.R[2] ) );
	}



	
	
	
	public Vector< Molecule > readBiomacromoleculesFromPDB( String file_name ){	
		Vector< Molecule > vMolecules = new Vector< Molecule >();
		
		Molecule mol = null;
		Atom atom = null;
		
		String line;
		double[] r = new double[3];

		FileInputStream fileinputstream;
		BufferedReader bufferedreader;	

		int atom_line_index = 0;

		try{
			fileinputstream = new FileInputStream( new File( file_name ) );
			bufferedreader = new BufferedReader( new InputStreamReader( fileinputstream ) );

			while( ( line = bufferedreader.readLine()) != null ){
				if( line.startsWith( "ATOM" ) && !line.substring( 12, 16 ).trim().startsWith( "H" ) ){
					atom_line_index++;		
					
					if( atom_line_index == 1 )
						mol = new Molecule();

					r[0] = Double.parseDouble( line.substring( 30, 38 ) );
					r[1] = Double.parseDouble( line.substring( 38, 46 ) );
					r[2] = Double.parseDouble( line.substring( 46, 54 ) );						
					
					atom = new Atom( line.substring( 12, 16 ).trim(), line.substring( 17, 20 ).trim(), line.substring( 21, 22 ), Integer.parseInt( line.substring( 22, 26 ).trim() ), line.substring( 22, 27 ).trim(), r, line );

					mol.atoms.add( atom );
				}
				else if( line.startsWith( "TER" ) ){
					vMolecules.add( mol );
					atom_line_index = 0;
				}	
				else if( line.startsWith( "END" ) )
					break;
			}	
			
			bufferedreader.close();
			fileinputstream.close();			
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}



		for( int i = 0 ; i < vMolecules.size() ; i++ ){
			if( vMolecules.elementAt(i) == null || vMolecules.elementAt(i).atoms.size() == 0 ){
				vMolecules.removeElementAt(i);
				i--;
			}
		}

		return vMolecules;		
	}
	
	
	
	
	
	
	public Vector< Molecule > readBiomacromoleculesFromPDBByChainId( String file_name ){	
		Vector< Molecule > vMolecules = new Vector< Molecule >();
		
		Molecule mol = null;
		Atom atom = null;
		
		String line;
		double[] r = new double[3];

		FileInputStream fileinputstream;
		BufferedReader bufferedreader;	

		char chain_id_prev = '0';
		char chain_id_curr = '0';
		int atom_line_index = 0;

		try{
			fileinputstream = new FileInputStream( new File( file_name ) );
			bufferedreader = new BufferedReader( new InputStreamReader( fileinputstream ) );

			while( ( line = bufferedreader.readLine()) != null ){
				if( line.startsWith( "ATOM" ) && !line.substring( 12, 16 ).trim().startsWith( "H" ) ){
					atom_line_index++;	
					
					chain_id_curr = line.charAt(21);
					
					if( chain_id_curr != chain_id_prev ){
						if( atom_line_index != 1 )
							vMolecules.add( mol );
							
						mol = new Molecule();
					}

					r[0] = Double.parseDouble( line.substring( 30, 38 ) );
					r[1] = Double.parseDouble( line.substring( 38, 46 ) );
					r[2] = Double.parseDouble( line.substring( 46, 54 ) );						
					
					atom = new Atom( line.substring( 12, 16 ).trim(), line.substring( 17, 20 ).trim(), line.substring( 21, 22 ), Integer.parseInt( line.substring( 22, 26 ).trim() ), line.substring( 22, 27 ).trim(), r, line );

					mol.atoms.add( atom );
					
					chain_id_prev = chain_id_curr;
				}
			}	
			
			vMolecules.add( mol );
			
			bufferedreader.close();
			fileinputstream.close();			
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}



		for( int i = 0 ; i < vMolecules.size() ; i++ ){
			if( vMolecules.elementAt(i) == null || vMolecules.elementAt(i).atoms.size() == 0 ){
				vMolecules.removeElementAt(i);
				i--;
			}
		}

		return vMolecules;		
	}
	
	
	
	
		
	
	public Vector< Molecule > readHeteromoleculesFromPDB( String file_name ){	
		Vector< Molecule > vMolecules = new Vector< Molecule >();
		
		Molecule mol = null;
		Atom atom = null;
		
		String line;
		double[] r = new double[3];
		
		FileInputStream fileinputstream;
		BufferedReader bufferedreader;	

		String chain_id_prev = "";
		String chain_id_current = "";
		
		int res_num_prev = 0;
		int res_num_current = 0;

		int hetero_line_index = 0;

		try{
		   fileinputstream = new FileInputStream( new File( file_name ) );
		   bufferedreader = new BufferedReader( new InputStreamReader( fileinputstream ) );

           while( ( line = bufferedreader.readLine()) != null ){
				if( line.startsWith( "HETATM" ) && !line.substring( 17, 20 ).equalsIgnoreCase( "HOH" ) && !line.substring( 12, 16 ).trim().startsWith( "H" ) ){
					hetero_line_index++;						

					chain_id_current = line.substring( 21, 22 );
					res_num_current = Integer.parseInt( line.substring( 22, 26 ).trim() );
					r[0] = Double.parseDouble( line.substring( 30, 38 ) );
					r[1] = Double.parseDouble( line.substring( 38, 46 ) );
					r[2] = Double.parseDouble( line.substring( 46, 54 ) );	

					atom = new Atom( line.substring( 12, 16 ).trim(), line.substring( 17, 20 ), line.substring( 21, 22 ), res_num_current, line.substring( 22, 27 ).trim(), r, line );

					if( hetero_line_index == 1 )
						mol = new Molecule();
					else if( hetero_line_index != 1 && ( !chain_id_current.equalsIgnoreCase( chain_id_prev ) || res_num_current != res_num_prev ) ){
						vMolecules.add( mol );

						mol = new Molecule();
					}
						
					mol.atoms.add( atom );					

					chain_id_prev = chain_id_current;
					res_num_prev = res_num_current;
				}
				else if( line.startsWith( "END" ) ){
					vMolecules.add( mol );
					break;
				}
			}			
			
			bufferedreader.close();
			fileinputstream.close();
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}


		for( int i = 0 ; i < vMolecules.size() ; i++ ){
			if( vMolecules.elementAt(i) == null || vMolecules.elementAt(i).atoms.size() == 0 ){
				vMolecules.removeElementAt(i);
				i--;
			}
		}



        // *** remove metal ions ***
		for( int i = 0 ; i < vMolecules.size() ; i++ ){
			if( vMolecules.elementAt(i).atoms.size() == 1 ){
				vMolecules.removeElementAt(i);
				i--;
			}
		}




		// *** merge multipart ligand & remove overlaped ligand ***
		double dist;
		int frag_1, frag_2;
		boolean isMultipartLigand = false;
		boolean isOverlapedLigand = false;

		for( int i = 0 ; i < vMolecules.size() ; i++ ){
			frag_1 = i;
			frag_2 = i + 1;			

			if( frag_2 == vMolecules.size() )
				break;

			isMultipartLigand = false;
			isOverlapedLigand = false;

			for( int a = 0 ; a < vMolecules.elementAt(frag_1).atoms.size() ; a++ ){
				for( int b = 0 ; b < vMolecules.elementAt(frag_2).atoms.size() ; b++ ){
					if( !isMetalIon( vMolecules.elementAt(frag_1).atoms.elementAt(a).atom_name ) && !isMetalIon( vMolecules.elementAt(frag_2).atoms.elementAt(b).atom_name ) ){
						dist = getDistance( vMolecules.elementAt(frag_1).atoms.elementAt(a), vMolecules.elementAt(frag_2).atoms.elementAt(b) );

						if( dist < 1.0 ){
							isOverlapedLigand = true;
							break;
						}

						if( dist >= 1.0 && dist <= 2.0 ){
							isMultipartLigand = true;
							break;
						}
					}
				}

				if( isOverlapedLigand || isMultipartLigand )
					break;
			}

			if( isMultipartLigand ){
				for( int b = 0 ; b < vMolecules.elementAt(frag_2).atoms.size() ; b++ ){
					atom = vMolecules.elementAt(frag_2).atoms.elementAt(b);
					vMolecules.elementAt(frag_1).atoms.add( atom );	
				}

				vMolecules.removeElementAt(frag_2);
				i--;	
			}				
			else if( isOverlapedLigand ){
				vMolecules.removeElementAt(frag_2);
				i--;	
			}	
		}


		for( int i = 0 ; i < vMolecules.size() ; i++ ){
			if( vMolecules.elementAt(i) == null || vMolecules.elementAt(i).atoms.size() == 0 ){
				vMolecules.removeElementAt(i);
				i--;
			}
		}

		
		return vMolecules;		
	}
	
	
	
	
	
	public boolean isConvalentlyLinkedLigand( Molecule protein, Molecule ligand ){
		boolean isCovalentlyLinked = false;
		double dist;
		int num_lig = ligand.atoms.size();
		int num_prot = protein.atoms.size();

		for( int i = 0 ; i < num_prot ; i++ ){
			for( int j = 0 ; j < num_lig ; j++ ){
				if( !isMetalIon( ligand.atoms.elementAt(j).atom_name ) ){
					dist = getDistance( protein.atoms.elementAt(i), ligand.atoms.elementAt(j) );

					if( dist >= 1.0 && dist <= 2.0 ){
						isCovalentlyLinked = true;

						return isCovalentlyLinked;
					}			
				}
			}					
		}

		return isCovalentlyLinked;
	}
	
	
	
		
	
	
	public Vector<Atom> readStructureFromPDB( String file_name ){	
		Vector<Atom> vMol = new Vector<Atom>();
		String line;
		double[] r = new double[3];
		Atom atom = null;

		FileInputStream fileinputstream;
		BufferedReader bufferedreader;	

		try{
			fileinputstream = new FileInputStream( new File( file_name ) );
			bufferedreader = new BufferedReader( new InputStreamReader( fileinputstream ) );

			while( ( line = bufferedreader.readLine()) != null ){
				if( ( line.startsWith( "ATOM" ) || line.startsWith( "HETATM" ) ) && !line.substring( 12, 16 ).trim().startsWith( "H" ) ){
					r[0] = Double.parseDouble( line.substring( 30, 38 ) );
					r[1] = Double.parseDouble( line.substring( 38, 46 ) );
					r[2] = Double.parseDouble( line.substring( 46, 54 ) );						
					
					atom = new Atom( line.substring( 12, 16 ).trim(), line.substring( 17, 20 ).trim(), line.substring( 21, 22 ), Integer.parseInt( line.substring( 22, 26 ).trim() ), r, line );

					vMol.add( atom );
				}
				else if( line.startsWith( "TER" ) || line.startsWith( "END" ) ){
					break;
				}
			}	
			
			bufferedreader.close();
			fileinputstream.close();			
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}		

		return vMol;		
	}
	
	
	
	
	
	public Vector<Atom> readAtomCoordinatesFromPDB( String file_name ){	
		Vector<Atom> vMol = new Vector<Atom>();
		String line;
		double[] r = new double[3];
		Atom atom = null;

		FileInputStream fileinputstream;
		BufferedReader bufferedreader;	

		try{
			fileinputstream = new FileInputStream( new File( file_name ) );
			bufferedreader = new BufferedReader( new InputStreamReader( fileinputstream ) );

			while( ( line = bufferedreader.readLine()) != null ){
				if( line.startsWith( "ATOM" ) ){
					r[0] = Double.parseDouble( line.substring( 30, 38 ) );
					r[1] = Double.parseDouble( line.substring( 38, 46 ) );
					r[2] = Double.parseDouble( line.substring( 46, 54 ) );						
					
					atom = new Atom( line.substring( 12, 16 ).trim(), line.substring( 17, 20 ).trim(), line.substring( 21, 22 ), Integer.parseInt( line.substring( 22, 26 ).trim() ), r, line );

					vMol.add( atom );
				}
			}	
			
			bufferedreader.close();
			fileinputstream.close();			
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}		

		return vMol;		
	}
	
	
	
		
			
	public Vector<Atom> readWatersFromPDB( String file_name ){	
		Vector<Atom> vMol = new Vector<Atom>();
		String line;
		double[] r = new double[3];
		Atom atom = null;

		FileInputStream fileinputstream;
		BufferedReader bufferedreader;	

		try{
			fileinputstream = new FileInputStream( new File( file_name ) );
			bufferedreader = new BufferedReader( new InputStreamReader( fileinputstream ) );

			while( ( line = bufferedreader.readLine()) != null ){
				if( line.startsWith( "HETATM" ) && line.substring( 17, 20 ).equalsIgnoreCase( "HOH" ) ){
					r[0] = Double.parseDouble( line.substring( 30, 38 ) );
					r[1] = Double.parseDouble( line.substring( 38, 46 ) );
					r[2] = Double.parseDouble( line.substring( 46, 54 ) );						
					
					atom = new Atom( line.substring( 12, 16 ).trim(), line.substring( 17, 20 ).trim(), line.substring( 21, 22 ), Integer.parseInt( line.substring( 22, 26 ).trim() ), r, line );

					vMol.add( atom );
				}
			}	
			
			bufferedreader.close();
			fileinputstream.close();			
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}		

		return vMol;		
	}
	
	
	
	
	
	
	// *** read crystal metal atoms ***
	public Vector<Atom> readMetalsFromPDB( String file_name ){	
		Vector<Atom> vMol = new Vector<Atom>();
		String line;
		double[] r = new double[3];
		Atom atom = null;

		FileInputStream fileinputstream;
		BufferedReader bufferedreader;	

		try{
			fileinputstream = new FileInputStream( new File( file_name ) );
			bufferedreader = new BufferedReader( new InputStreamReader( fileinputstream ) );

			while( ( line = bufferedreader.readLine()) != null ){
				if( line.startsWith( "HETATM" ) && ( line.charAt(77) != 'H' ) ){
					if( isMetalIon( line.substring( 76, 78 ).trim() ) ){
						r[0] = Double.parseDouble( line.substring( 30, 38 ) );
						r[1] = Double.parseDouble( line.substring( 38, 46 ) );
						r[2] = Double.parseDouble( line.substring( 46, 54 ) );						
					
						atom = new Atom( line.substring( 12, 16 ).trim(), line.substring( 17, 20 ).trim(), line.substring( 21, 22 ), Integer.parseInt( line.substring( 22, 26 ).trim() ), r, line );

						vMol.add( atom );
					}
				}
			}	
			
			bufferedreader.close();
			fileinputstream.close();			
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}		

		return vMol;		
	}
	


	

	public void removeDNAAndRND( Vector< Molecule > vMolecules ){
		String res;

		for( int i = 0 ; i < vMolecules.size() ; i++ ){
			res = vMolecules.elementAt(i).atoms.elementAt(0).residue_name;

			if( res.equalsIgnoreCase( "DA" ) || res.equalsIgnoreCase( "DC" ) || res.equalsIgnoreCase( "DG" ) || res.equalsIgnoreCase( "DT" ) ||
				res.equalsIgnoreCase( "A" ) || res.equalsIgnoreCase( "C" ) || res.equalsIgnoreCase( "G" ) || res.equalsIgnoreCase( "U" ) ){
				//System.out.println( "is DNA or RNA" );
				vMolecules.removeElementAt(i);
				i--;
			}
		}
	}
	
	
	
	
	
	public void removeNonInteractingProteinChains( Vector< Molecule > vLigands, Vector< Molecule > vProteins, double cutoff ){
		int num_atoms_receptor;
		double min_dist = 10000;
		double dist;
		
		for( int i = 0 ; i < vProteins.size() ; i++ ){
			num_atoms_receptor = vProteins.elementAt(i).atoms.size();
			min_dist = 10000;
			
			for( int l = 0 ; l < vLigands.size() ; l++ ){
				int num_atoms_ligand = vLigands.elementAt(l).atoms.size();
				
				for( int j = 0 ; j < num_atoms_ligand ; j++ ){
					 for( int k = 0 ; k < num_atoms_receptor ; k++ ){
				 		dist = getDistance( vLigands.elementAt(l).atoms.elementAt(j).R, vProteins.elementAt(i).atoms.elementAt(k).R );
				 	
				 		if( dist < min_dist ){
				 			min_dist = dist;
				 		}
				 	}
				}
			}
			
			if( min_dist > cutoff ){
				vProteins.removeElementAt(i);
				i--;
			}
		}
	}
	
	
	
	
	public void removeNonInteractingAtomsWithMolecule( Vector<Atom> vAtoms, Vector<Atom> vMolecule, double cutoff ){
		int num_atoms_mol = vMolecule.size();
		double min_dist = 10000;
		char min_atom = 'C';
		double dist;
					
		for( int i = 0 ; i < vAtoms.size() ; i++ ){
			min_dist = 10000;
			 
			for( int j = 0 ; j < num_atoms_mol ; j++ ){
				 dist = getDistance( vAtoms.elementAt(i).R, vMolecule.elementAt(j).R );
				 	
				 if( dist < min_dist ){
				 	min_dist = dist;
				 	min_atom = vMolecule.elementAt(j).atom_name.charAt(0);
				}
			}
			 
			if( ( min_dist > cutoff ) || ( ( min_dist <= cutoff ) && ( min_atom == 'C' ) ) ){
				vAtoms.removeElementAt(i);
				i--;
			}
		}
	}
	
	
	
	
	
	
		
		
    // Metal ions were defined based on MetalPDB Statistics (http://metalweb.cerm.unifi.it/statistics/summary/)
	public boolean isMetalIon( String atom_name ){
		if( atom_name.equalsIgnoreCase( "FE" ) || atom_name.equalsIgnoreCase( "CU" ) || atom_name.equalsIgnoreCase( "CD" ) ||
			atom_name.equalsIgnoreCase( "K" )  || atom_name.equalsIgnoreCase( "MN" ) || atom_name.equalsIgnoreCase( "NA" ) || 
			atom_name.equalsIgnoreCase( "ZN" ) || atom_name.equalsIgnoreCase( "CA" ) || atom_name.equalsIgnoreCase( "MG" ) || 
			atom_name.equalsIgnoreCase( "CO" ) || atom_name.equalsIgnoreCase( "NI" ) || atom_name.equalsIgnoreCase( "U" )  ||
			atom_name.equalsIgnoreCase( "LI" ) || atom_name.equalsIgnoreCase( "BE" ) || atom_name.equalsIgnoreCase( "AL" ) ||
			atom_name.equalsIgnoreCase( "SC" ) || atom_name.equalsIgnoreCase( "V" )  || atom_name.equalsIgnoreCase( "CR" ) ||
			atom_name.equalsIgnoreCase( "LI" ) || atom_name.equalsIgnoreCase( "BE" ) || atom_name.equalsIgnoreCase( "AL" ) ||
			atom_name.equalsIgnoreCase( "GA" ) || atom_name.equalsIgnoreCase( "RB" ) || atom_name.equalsIgnoreCase( "SR" ) ||
			atom_name.equalsIgnoreCase( "Y" )  || atom_name.equalsIgnoreCase( "ZR" ) || atom_name.equalsIgnoreCase( "MO" ) ||
			atom_name.equalsIgnoreCase( "RU" ) || atom_name.equalsIgnoreCase( "RH" ) || atom_name.equalsIgnoreCase( "PD" ) ||
			atom_name.equalsIgnoreCase( "AG" ) || atom_name.equalsIgnoreCase( "IN" ) || atom_name.equalsIgnoreCase( "SN" ) ||			
			atom_name.equalsIgnoreCase( "SB" ) || atom_name.equalsIgnoreCase( "CS" ) || atom_name.equalsIgnoreCase( "BA" ) ||
			atom_name.equalsIgnoreCase( "LA" ) || atom_name.equalsIgnoreCase( "CE" ) || atom_name.equalsIgnoreCase( "PR" ) ||			
			atom_name.equalsIgnoreCase( "SM" ) || atom_name.equalsIgnoreCase( "EU" ) || atom_name.equalsIgnoreCase( "GD" ) ||
			atom_name.equalsIgnoreCase( "TB" ) || atom_name.equalsIgnoreCase( "DY" ) || atom_name.equalsIgnoreCase( "HO" ) ||
			atom_name.equalsIgnoreCase( "ER" ) || atom_name.equalsIgnoreCase( "YB" ) || atom_name.equalsIgnoreCase( "LU" ) ||
			atom_name.equalsIgnoreCase( "HF" ) || atom_name.equalsIgnoreCase( "TA" ) || atom_name.equalsIgnoreCase( "W" )  ||
			atom_name.equalsIgnoreCase( "RE" ) || atom_name.equalsIgnoreCase( "OS" ) || atom_name.equalsIgnoreCase( "IR" ) ||
			atom_name.equalsIgnoreCase( "PT" ) || atom_name.equalsIgnoreCase( "AU" ) || atom_name.equalsIgnoreCase( "HG" ) ||
			atom_name.equalsIgnoreCase( "TL" ) || atom_name.equalsIgnoreCase( "PB" ) || atom_name.equalsIgnoreCase( "BI" ) ||
			atom_name.equalsIgnoreCase( "PA" ) )
			return true;
		else 
			return false;
	}



			
	
	
	
	public void selectContactingReceptorResidues( Vector< Atom > ligand, Molecule receptor, double cutoff_dist ){
		int num_atoms_lig = ligand.size();
		int num_atoms_rec = receptor.atoms.size();

		double dist;
		String value = null;

		for( int i = 0 ; i < num_atoms_lig ; i++ ){
			for( int j = 0 ; j < num_atoms_rec ; j++ ){
				dist = getDistance( ligand.elementAt(i), receptor.atoms.elementAt(j) );

				if( dist <= cutoff_dist ){
					value = receptor.htResIndex.get( receptor.atoms.elementAt(j).sequence_number_nr );
					receptor.isSelectedResidue[Integer.parseInt(value)] = true;
				}
			}
		}
	}



	

	public void writeSelectedResiduesInPDBFormat( Molecule receptor, String file_name ){
		int num_atoms = receptor.atoms.size();
		String value = null;

		try{ 
			DataOutputStream out = new DataOutputStream( new FileOutputStream( new File( file_name ) ) );	

			for( int i = 0 ; i < num_atoms ; i++ ){
				value = receptor.htResIndex.get( receptor.atoms.elementAt(i).sequence_number_nr );
				
				if( receptor.isSelectedResidue[Integer.parseInt(value)] ){
					out.writeBytes( receptor.atoms.elementAt(i).pdb_line );	
					out.writeBytes( "\n" );
				}
			}			

			out.writeBytes( "TER" );

			out.close();	
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}
	}
	
	
	
		
	
	
	public void writeMoleculeInPDBFormat( Vector<Atom> vAtoms, String file_name ){
		int num_atoms = vAtoms.size();

		try{ 
			DataOutputStream out = new DataOutputStream( new FileOutputStream( new File( file_name ) ) );	

			for( int i = 0 ; i < num_atoms ; i++ ){
				out.writeBytes( vAtoms.elementAt(i).pdb_line );				
				out.writeBytes( "\n" );
			}

			out.writeBytes( "TER" );

			out.close();	
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}
	}
	
	
		
	
	
	public int getNumResidues( Molecule protein ){
		int num_atoms = protein.atoms.size();
		int num_res = 0;
		
		String curr_resSeq = "XXX";
		String prev_resSeq = "XXX";
		
		for( int i = 0 ; i < num_atoms ; i++ ){
			curr_resSeq = protein.atoms.elementAt(i).sequence_number_nr;
			
			if( !prev_resSeq.equalsIgnoreCase( curr_resSeq ) )
				num_res++;
			
			prev_resSeq = curr_resSeq;
		}
		
		return num_res;
	}
	
	
	
	
	
	public static int getNumResidues( String struct ){
		FileInputStream struct_stream;
		BufferedReader struct_reader;	
		String struct_line;	
		int num_res = 0;
				
		try{
			struct_stream = new FileInputStream( new File( struct ) );
			struct_reader = new BufferedReader( new InputStreamReader( struct_stream ) );	

			while( ( struct_line = struct_reader.readLine()) != null ){
				if( struct_line.startsWith( "ATOM" ) && struct_line.substring( 13, 15 ).trim().equalsIgnoreCase( "CA" ) )
					num_res++;
			}

			struct_reader.close();
			struct_stream.close();		
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}	
		
		return num_res;	
	}
	
	
	
	
	
	public Vector<Atom> duplicateAtomVector( Vector<Atom> vAtoms ){
		Vector<Atom> vDuplicate = new Vector<Atom>();
		
		for( int i = 0 ; i < vAtoms.size() ; i++ ){
			vDuplicate.add( vAtoms.elementAt(i) );
		}
		
		return vDuplicate;
	}
	
	
	
	
	
	public String makePDBline( String line, double[] R ){	
		String new_line = "";
		String line_1 = line.substring( 0, 30 );
		String line_2 = line.substring( 54, line.length() );

		for( int i = 0 ; i < 3 ; i++ ){
			if( format.format( R[i] ).equalsIgnoreCase( "-0.000" ) )
				R[i] = Double.parseDouble( format.format( 0.000 ) );
			else
				R[i] = Double.parseDouble( format.format( R[i] ) );

			if( R[i] >= 0 ){
				if( R[i] / 10 < 1 )
					new_line = new_line + "   " + format.format( R[i] );
				else if( R[i] / 10 >= 1 && R[i] / 10 < 10 )
					new_line = new_line + "  " + format.format( R[i] );
				else
					new_line = new_line + " " + format.format( R[i] );
			}
			else{
				if( R[i] / 10 > -1 )
					new_line = new_line + "  " + format.format( R[i] );
				else if( R[i] / 10 <= -1 && R[i] / 10 > -10 )
					new_line = new_line + " " + format.format( R[i] );
				else
					new_line = new_line + format.format( R[i] );
			}
		}	

		new_line = line_1 + new_line + line_2;

		return new_line;
	}
}
